//
//  AmberView.h
//  Amber-X
//
//  Created by Kenneth Hoffmann on 10/18/04.
//  Copyright 2004 __MyCompanyName__. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@class TaskWrap;
@class AmberModel;
@class InputPanel;
@class SoxHandler;
@class GraphView;
@class MouseWindow;
@class SecondModel;

@interface AmberView : NSObject {
	NSMutableArray * flags;
	
	int morph;
	bool inputOpen;
	bool mainOpen;
	
	/* GUI Elements */
	IBOutlet NSSlider * initGrainSize;
	IBOutlet NSSlider * finalGrainSize;
	IBOutlet NSSlider * initDensity;
	IBOutlet NSSlider * finalDensity;
	IBOutlet NSTextField * initGrainSizeField;
	IBOutlet NSTextField * finalGrainSizeField;
	IBOutlet NSTextField * initDensityField;
	IBOutlet NSTextField * finalDensityField;
	
	IBOutlet NSTextField * outputFileLength;
	IBOutlet NSTextField * windowSize;
	IBOutlet NSButton * scaleSwitch;
	IBOutlet NSTextView * textBox;
	
	IBOutlet NSPopUpButton * bitDepth;
	
	IBOutlet NSButton * morphingSwitch;
	IBOutlet NSTextField * morphStart;
	IBOutlet NSPopUpButton * morphStartType;
	IBOutlet NSTextField * morphEnd;
	IBOutlet NSPopUpButton * morphEndType;
	
	IBOutlet NSTextField * envelope;
	IBOutlet NSPopUpButton * envelopeType;
	
	IBOutlet NSPopUpButton * grainGeneratorType;
	
	IBOutlet NSButton * executeTaskButton;
	IBOutlet NSButton * cancelTaskButton;
	
	IBOutlet NSProgressIndicator * indicator;
	
	IBOutlet id outputField;
	IBOutlet NSMenu * grainType;
	
	NSString * tempPath; //Temporary path for output, pre-conversion to WAV
	
	///Used by preview and delete. Path to point to the file to preview, bool to make sure a task has run
	NSString * outputPath;
	bool taskHasRun;
	IBOutlet NSMovieView * previewView;
	NSMovie * mov;
	IBOutlet NSButton * previewButton;
	IBOutlet NSButton * deleteButton;
	
	IBOutlet NSDrawer * drawer;
	
	/* Connect to the class responsible for running the NSTask */
	IBOutlet TaskWrap * task;
	
	/* Conversions */
	IBOutlet SoxHandler * soxHandle;
	
	IBOutlet NSWindow * mainWindow;
	NSWindowController * mainController;
	
	/* Input Table & Window */
	IBOutlet NSTableView * inputTable;
	IBOutlet InputPanel * inputPanel;
	NSWindowController * inputController;
	
	NSMutableArray * inputFiles;
	NSString * savePath;
	
	/* The path that was last opened/saved. Used by Save and Revert */
	NSString * settingsPath;
	
	NSNotificationCenter * defaultCenter;
	
	/* PREFERENCES */
	IBOutlet NSWindow * preferencesWindow;
	NSWindowController * prefController;
	int convert;
	bool prefOpen;
	IBOutlet NSButton * convertSwitch;
	
	/* GRAIN SIZE FUNCTIONS */
	IBOutlet MouseWindow * grainSizeWindow;
	NSWindowController * grainController;
	IBOutlet NSButton * grainSwitch;
	IBOutlet GraphView * grainView;
	bool grainCurrentlyOpen;
	bool grainHasBeenSaved;
	bool useGrainFunction;
	NSData * grainData;
	NSString * grainPath;
	
	/* GRAIN DENSITY FUNCTIONS */
	IBOutlet MouseWindow * grainDensityWindow;
	NSWindowController * densityController;
	IBOutlet NSButton * densitySwitch;
	IBOutlet GraphView * densityView;
	bool densityCurrentlyOpen;
	bool densityHasBeenSaved;
	bool useDensityFunction;
	NSData * densityData;
	NSString * densityPath;
	
	NSUserDefaults * defaults;
	
}

/* Determines output path */
-(IBAction)chooseSavePath:(id)sender;
/* Loads an array of arguments and passes it to TaskWrap */
-(IBAction)runAmber:(id)sender;
/* Mops up the task */
-(void)finishTask;
-(void)finishTaskSecondPart;
/* Cancels the Task */
-(IBAction)cancelTask:(id)sender;
/* Switches the Enabled states of the Run Amber and Cancel Task buttons. One is enabled, one disabled. */
-(void)switchButtonStates;
////CONVERSION
-(void)outputDone:(BOOL)result;

/* If the user Closes the main Amber window, this re-opens it */
-(IBAction)openMainWindow:(id)sender;
/* Activated if the user closes the window. Makes sure that if they do open while a window is already open
	a second doesn't open */
-(void)mainWindowClose:(NSNotification *)aNotification;

/* Toggles whether or not to morph envelopes */
-(IBAction)toggleMorphing:(id)sender;
/* Function toggles */
-(IBAction)sizeDidToggle:(id)sender;
-(IBAction)densityDidToggle:(id)sender;

/* All these classes update text fields when their corresponding sliders change */
-(IBAction)initGrainSizeDidChange:(id)sender;
-(IBAction)finalGrainSizeDidChange:(id)sender;
-(IBAction)initDensityDidChange:(id)sender;
-(IBAction)finalDensityDidChange:(id)sender;

/* Sets the progress bar to val */
-(void)setIndict:(double)val;

/* Screens arguments for invalid values */
-(BOOL)checkArguments;
/* Rounds the double values sent to the grain text fields by the NSSliders */
-(NSString *)roundDouble:(NSString *)val;
/* Sets the save settings path in such a way to avoid memory leaks */
-(void)setSavePath:(NSString *)newPath;
/* Checks to see if the output path will overwrite something */
-(BOOL)checkSaveOverwrite:(NSString *)path;

///PREVIEW & DELETE
-(IBAction)previewSound:(id)sender;
-(IBAction)deleteSound:(id)sender;
/* Delegate method for drawer. Prevents user resizing it */
-(NSSize)drawerWillResizeContents:(NSDrawer *)sender toSize:(NSSize)contentSize;

///SAVING/LOADING
/* openBody and saveBody do all the work. SaveSettings and Open use file dialog boxes to get paths.
	Save and Revert use the settingsPath to save/open without asking for a path. */
-(IBAction)saveSettings:(id)sender;
-(IBAction)openSettings:(id)sender;
-(IBAction)revert:(id)sender;
-(IBAction)save:(id)sender;
-(void)saveBody:(NSString *)path;
-(void)openBody:(NSString *)path;

///INPUT PANEL METHODS
/* Opens the input table and registers it for dragging */
-(IBAction)chooseInputFile:(id)sender;
/* Adds a file to the table */
-(IBAction)addFile:(id)sender;
/* Removes from table */
-(IBAction)removeFile:(id)sender;
/* Close the input window */
-(IBAction)closeWindow:(id)sender;
-(void)addFilesToArray:(NSArray *)new;
/* Sets inputOpen to false */
-(void)inputPanelClose;

////GRAIN SIZE (FUNCTION)
-(IBAction)displayGrainSizeWindow:(id)sender;
-(IBAction)saveGrainData:(id)sender;
-(IBAction)cancelGrainData:(id)sender;

/////GRAIN DENSITY (FUNCTION)
-(IBAction)displayGrainDensityWindow:(id)sender;
-(IBAction)saveDensityData:(id)sender;
-(IBAction)cancelDensityData:(id)sender;

//PREFERENCES
-(IBAction)savePreferences:(id)sender;
-(IBAction)cancelPreferences:(id)sender;
-(IBAction)openPreferences:(id)sender;
-(void)prefClose;

/* Turns all the menu options into their corresponding ints */
-(NSString *)resolveBitMenu;
-(NSString *)resolveMorphStart;
-(NSString *)resolveMorphEnd;
-(NSString *)resolveEnvelopeType;
-(NSString *)resolveGrainType;

-(void)createTempFolder;
-(NSString *)getOutputPath;
-(void)printToTextBox:(NSString *)print;
@end
